#include <stdio.h>
#include "../../emu/h/types.h"
#include "../../emu/h/emuFileHandling.h"
#include "../../nes/h/nesGeneral.h"
#include "../../emu/h/emuGeneral.h"

extern char szFilename[260];
const char stateFolder [] = "states/";

int emuSaveState(FILE *f);
int emuLoadState(FILE *f);

/*
	10 ram
	11 cpu registers
	
	20 ppu registers
	21 sprram
	22 palette

	40 input regs
	50 wram size
	51 wram
	52 nametable flags
	53 nametable 0
	54 nametable 1

	60 APU error
	70 Mapper error
	71 not 128 byte mapper array
	72 couldnt load map array
*/

void StateError(int errNum)
{
	switch (errNum)
	{
	case 0:														break;
	case 10:	emuMsg("Error ram (QuickState)");				break;
	case 11:	emuMsg("Error cpu regs (QuickState)");			break;

	case 20:	emuMsg("Error ppu regs (QuickState)");			break;
	case 21:	emuMsg("Error sprram (QuickState)");			break;
	case 22:	emuMsg("Error palette (QuickState)");			break;

	case 40:	emuMsg("Error input regs (QuickState)");		break;
	case 50:	emuMsg("Error wram size (QuickState)");			break;
	case 51:	emuMsg("Error wram (QuickState)");				break;
	case 52:	emuMsg("Error nametable flags (QuickState)");	break;
	case 53:	emuMsg("Error nametable 0 (QuickState)");		break;
	case 54:	emuMsg("Error nametable 1 (QuickState)");		break;
	case 55:	emuMsg("Error chrram (QuickState)");			break;

	case 60:	emuMsg("Error apu (QuickState)");				break;
	case 70:	emuMsg("Error mapper (QuickState)");			break;
	case 71:	emuMsg("Error 71 flags (QuickState)");			break;
	case 72:	emuMsg("Error 72 0 (QuickState)");				break;

	default:
		emuMsg("unknown return error from QuickState");
		break;
	}
}

void saveState()
{
	char szPath[260], szExt[10]; FILE *f;

	if (!imageLoaded)
	{
		emuMsg("No image loaded (Save State Error)");
		return;
	}

	if (!szFilename[0])
	{
		emuMsg("No filename (Save State Error)");
		return;
	}

	sprintf(szExt, "ss%d.rem", StateNum);
	
	GetLocalPath(stateFolder, szExt, szPath, sizeof(szPath));
	f = fopen(szPath, "wb");
	if (!f)
	{
		emuMsg("Couldn't create/open state file");
		return;
	}

	StateError(emuSaveState(f));

	fclose(f);
	return;
}

void loadState()
{
	char szPath[260], szExt[10]; FILE *f;

	if (!imageLoaded)
	{
		emuMsg("No image loaded (load State Error)");
		return;
	}

	if (!szFilename[0])
	{
		emuMsg("No filename (load State Error)");
		return;
	}

	sprintf(szExt, "ss%d.rem", StateNum);
	
	GetLocalPath(stateFolder, szExt, szPath, sizeof(szPath));
	f = fopen(szPath, "rb");
	if (!f)
		return;

	StateError(emuLoadState(f));
	fclose(f);
	return;
}
