#include "includeSdl.h"
#include <stdio.h>
#include <string.h>
#include "../../emu/h/types.h"
#include "../../nes/h/nesGeneral.h"
#include "../../nes/h/cpu.h"
#include "../../emu/h/emuFileHandling.h"
#include "main.h"

#include "../pc/pcFS.h"
#include "../pc/physfs.h"

#ifdef WIN32
	#include <windows.h>
#endif

char quitApp=0;
static char guiMode=0;
const char wndName[] = "Reminesce";

TTF_Font *myFont;
extern SDL_Surface *screen;

enum { fontSize = 14 };

#ifdef WIN32
	const char fontPath[] = "/FONTS/COUR.TTF";
#else
	const char fontPath[] = "/usr/share/fonts/COUR.TTF";
#endif

enum
{
	numOfLinesOnscreen = 15
};


int filenameCursor=0, filenameOffset=0;
void saveState();
void loadState();
void drawList1();

void Msg(char *sz)
{
	#ifdef WIN32
		MessageBox(NULL, sz, wndName, MB_OK);
	#else
		printf(sz);
	#endif
}

void emuMsg(char *msg)
{
	Msg(msg);
}

void closeApp()
{
	stopSound();
	nesMode &=  (-1 ^ resetLoop);
	brunNes = 0;
	//destory any windows
	quitApp = 1;
}


void fileCursorKeyPress(int dir)
{
	int index = filenameCursor + filenameOffset;
	switch (dir)
	{
	case 1:
		if (index)
			index--;
		else
			index = getFSListAmount()-1;
		break;
	case 2:
		if (index < getFSListAmount()-1)
			index++;
		else 
			index = 0;
		break;
	case 3:
		if (index  > numOfLinesOnscreen)
			index -= numOfLinesOnscreen;
		else
			index = 0;
		break;
	case 4:
		if (index < getFSListAmount()-numOfLinesOnscreen - 1)
			index += numOfLinesOnscreen;
		else 
			index = getFSListAmount()-1;
		break;
	}
	filenameCursor = index % numOfLinesOnscreen;
	filenameOffset = index - filenameCursor;
}



void runTheNes()
{
runNesAgain:
	if (!brunNes)
		return;

	runNes();
	if (nesMode & resetLoop)
	{
		nesMode ^= resetLoop;
		brunNes = 1;
		goto runNesAgain;
	}
	SDL_WM_SetCaption(wndName, 0);
}

void keyDownEvent(SDL_Event *event)
{
	if (guiMode != 0)
	{
		switch (event->key.keysym.sym)
		{
		case SDLK_ESCAPE:
			closeApp();
			break;
		case SDLK_F1:
			brunNes = 1;
			runTheNes();
			break;

		case SDLK_F2:
			brunNes = 0;
			break;
		case SDLK_F5:
			saveState();
			break;
		case SDLK_F7:
			loadState();
			break;

		case SDLK_F12:
			SaveWram();
			break;

		case SDLK_o:
			if (event->key.keysym.mod & KMOD_LCTRL)
			{
				guiMode = 0;
				brunNes = 0;
			}
		case SDLK_r:
			if (event->key.keysym.mod & KMOD_LCTRL)
				ResetNes();
			break;
		case SDLK_f:
			if (event->key.keysym.mod & KMOD_LCTRL)
				HardResetNes();
			break;
		case SDLK_l:
			if (event->key.keysym.mod & KMOD_LCTRL)
				loadPath();
			break;
		default:
			if ( (SDLK_0 <= event->key.keysym.sym) && (event->key.keysym.sym <= SDLK_9) )
				StateNum = event->key.keysym.sym - SDLK_0;
		}
	}
	else
	{
		switch (event->key.keysym.sym)
		{
		case SDLK_ESCAPE:
			closeApp();
			break;
		case SDLK_UP:
			fileCursorKeyPress(1);
			break;
		case SDLK_DOWN:
			fileCursorKeyPress(2);
			break;
		case SDLK_LEFT:
			fileCursorKeyPress(3);
			break;
		case SDLK_RIGHT:
			fileCursorKeyPress(4);
			break;

		case SDLK_RETURN:
			{
				char sz[300];
				if ( FSenter(filenameCursor, sz) )
				{
					strcpy(szFilename, sz);
					guiMode = 1;		//loaded a file
					loadPath();
					SDL_FillRect(screen, 0, 0);
					SDL_Flip(screen);
				}
			}
			filenameOffset = filenameCursor = 0;
			break;
		}
	}
}

void RefreshGUI_Event(SDL_Event *event)
{
	switch( event->type ) 
	{
		case SDL_QUIT:
			printf("Got quit event\n");
			closeApp();
			break;

		case SDL_KEYDOWN:
			keyDownEvent(event);
			break;
	}
}

void RefreshGUI()
{
	SDL_Event event;
	SDL_PollEvent(&event);
	RefreshGUI_Event(&event);
}

void mainLoop()
{
	SDL_Event event;

	while (quitApp == 0)
	{
		if (guiMode == 0)
			drawList1();
		SDL_WaitEvent(&event);
		RefreshGUI_Event(&event);
	}
}


///////////////////////////////////////////////////////////////




void drawList1()
{
	SDL_Surface *TextBmp;
	SDL_Color c;
	int indx, indx2, i, numOfFiles;
	char *sz;

	c.unused = c.r = 0;
	c.g = 128;
	c.b = 255;

	SDL_Rect rFront;
	SDL_FillRect(screen, 0, 0);

	if (numOfFiles = getFSListAmount())
	{
		rFront.x = rFront.y = 0;
		TextBmp = TTF_RenderText_Solid(myFont, "->", c);
		rFront.y = filenameCursor * 15;
		SDL_BlitSurface(TextBmp, 0, screen, &rFront);
		SDL_FreeSurface(TextBmp);
	}

	if (FS_string == 0)
		goto end_drawList1;

	rFront.x = 20;
	for (indx = indx2 = 0; ((indx + filenameOffset) < numOfFiles) && (indx < numOfLinesOnscreen); indx++)
	{
		sz = FS_string[indx + filenameOffset];
		i = strlen(sz) - 3;
		while (sz[i] != 0)
			sz[i++] = sz[i] | 0x20;		//lower case it
		
		
		if ( TextBmp = TTF_RenderText_Solid(myFont, sz, c) )
		{
			rFront.y = indx2 * 15;
			SDL_BlitSurface(TextBmp, 0, screen, &rFront);
			SDL_FreeSurface(TextBmp);
		}
		indx2++;
	}
end_drawList1:
	SDL_Flip(screen);
}

int initFont()
{
	if(TTF_Init() == -1)
		return 1;

#ifdef WIN32
	{
		char sz[260];
		GetEnvironmentVariable("windir", sz, 260);
		strcat(sz, fontPath);
		myFont = TTF_OpenFont( sz, fontSize);
	}
#else
	myFont = TTF_OpenFont( fontPath, fontSize);
#endif
	
	if (myFont == 0)
		return 2;

	return 0;
}

void uninitFont()
{
	if ( myFont )
	{
		TTF_CloseFont(myFont);
		myFont = 0;
	}
	TTF_Quit();
}

int initGUI()
{
	if ( initFont() )
		return 1;
	else
		return 0;
}

void uninitGUI()
{
	uninitFont();
}
