
#include "../../emu/h/types.h"

#ifdef noSoundPlz

int initSound() {
	return 0;
}

void uninitSound() {
}

void playSound() {
}

void stopSound() {
}

#else

#include "includeSdl.h"
#include "../../nes/blargg/Nes_Apu.h"
#include "../../nes/blargg/Sound_Queue.h"
#include "../../nes/h/apu.h"
#include "../../emu/h/interface.h"
#include "../../emu/h/mapper.h"

const int callbackSize = 1024 * 2;			// sound callback buffer size
const int queueTarget = callbackSize;		// queue is kept filled to this size
Sound_Queue queue( queueTarget * 2 );		// keep room for extra sound
BOOST_STATIC_ASSERT( callbackSize <= 1024 * 2 );

static char inSoundCode = 0;
static char bInitSound = 0;

void soundCallback(void *userdata, Uint8 *stream, int len)
{
	int count = len / 2;
	if (inSoundCode && queue.avail() >= count)
		queue.read((short*)stream, count ); // read from queue only if enough to fill buffer
	else
		memset(stream, 0, len);
}

int initSound()
{
	int a = callbackSize;
	bInitSound=1;
	
	SDL_AudioSpec spec;
	spec.freq		= samples_per_sec;
	spec.format		= AUDIO_S16SYS; // signed 16-bit in native endian
	spec.channels	= 1;
	spec.samples	= callbackSize;
	spec.callback	= soundCallback;
	spec.userdata	= NULL;
	
	// Open the audio device with automatic conversion if hardware
	// doesn't support requested format
	if ( SDL_OpenAudio(&spec, NULL) < 0 )
	{
	  printf("Couldn't open audio: %s\n", SDL_GetError());
	  return 1;
	}

	/* Prepare callback for playing */
	/* Start playing */
	SDL_PauseAudio(0);
	return 0;
}

void uninitSound()
{
	// Nothing needed
}

void stopSound()
{
	inSoundCode = 0;
}

#include <math.h>
static void make_sine( short* out, long count )
{
    static double angle;
	while ( count-- )
		*out++ = 0x2000 * sin( angle += 0.04 );
}

void playSound()
{
	inSoundCode = 1;
	
	const int temp_size = samples_per_sec / 30;
	static blip_sample_t temp_buf [temp_size];
	
	long count = blipBuf.read_samples( temp_buf, temp_size );
	if (mapInfo->MapperSnd)
		mapInfo->MapperSnd((s16*) temp_buf, count);
	
	// Generate exactly 735 samples of sine wave, to find if bug is in emulation
	// or the sound callback.
	//count = 735;
	//make_sine( write_buf, count );
	
	queue.write( temp_buf, count );
	
	// Wait if queue has accumulated too many samples
	while ( queue.avail() > queueTarget ) { }
}

#endif

